//
//  BMKPanoViewController.m
//  BaiduRecommendStopDemo
//
//  Created by v_wangdachuan on 2022/11/15.
//

#import "BMKPanoViewController.h"
#import <BaiduPanoSDK/BaiduPanoramaView.h>
#import <BaiduPanoSDK/BaiduPanoUtils.h>
#import "BMKCustomPointAnnotation.h"
#import "BaseNavigationController.h"


@interface BMKPanoViewController ()<BaiduPanoramaViewDelegate, BMKMapViewDelegate, BMKLocationManagerDelegate, BMKGeoCodeSearchDelegate>
@property (strong, nonatomic) BaiduPanoramaView  *panoramaView;

@property (nonatomic, strong) BMKMapView *mapView;

@property (nonatomic, strong) BMKLocationManager *locManager;

@property (nonatomic, strong) BMKUserLocation *userLocation;

@property (nonatomic, strong) BMKCustomPointAnnotation *pointAnnotation;

@property (nonatomic, strong) UIButton *returnMapBtn;
@end

@implementation BMKPanoViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
    [self configUI];
   
    [self.locManager startUpdatingLocation];
    [self.locManager startUpdatingHeading];
}

- (void)configUI {
    [self.view addSubview:self.mapView];
    [self.mapView addAnnotation:self.pointAnnotation];
    
    UIButton *panoBtn = [UIButton buttonWithType:UIButtonTypeCustom];
    panoBtn.layer.masksToBounds = YES;
    panoBtn.frame = CGRectMake(CGRectGetWidth(self.view.frame) - 50, CGRectGetMinY(self.mapView.frame) + 15, 30, 30);
    panoBtn.layer.cornerRadius = 15;
    panoBtn.backgroundColor = [UIColor blackColor];
    panoBtn.alpha = 0.6;
    [panoBtn setImage:[UIImage imageNamed:@"close"] forState:UIControlStateNormal];
    [panoBtn addTarget:self action:@selector(btnClick) forControlEvents:UIControlEventTouchUpInside];
    [self.view addSubview:panoBtn];
}

- (void)mapBtnClick {
    self.returnMapBtn.hidden = !self.returnMapBtn.hidden;
    [self.panoramaView setFrame:CGRectMake(0, -280, CGRectGetWidth(self.view.frame), CGRectGetHeight(self.view.frame))];
}

- (void)btnClick {
    self.returnMapBtn.hidden = !self.returnMapBtn.hidden;
    [self.panoramaView setFrame:self.view.bounds];
}

- (void)setCoordinate:(CLLocationCoordinate2D)coordinate {
    _coordinate = coordinate;
    [self customPanoView];
    [self.mapView setCenterCoordinate:_coordinate animated:YES];
}

- (void)customPanoView {
    
    CGRect frame = CGRectMake(0, -280, CGRectGetWidth(self.view.frame), CGRectGetHeight(self.view.frame));
    
    // key 为在百度LBS平台上统一申请的接入密钥ak 字符串
    self.panoramaView = [[BaiduPanoramaView alloc] initWithFrame:frame key:@"输入您的AK"];
    [self.panoramaView setAgreePrivacy:YES];
    // 为全景设定一个代理
    self.panoramaView.delegate = self;
    [self.view addSubview:self.panoramaView];
    // 设定全景的清晰度， 默认为middle
    [self.panoramaView setPanoramaImageLevel:ImageDefinitionMiddle];
    // 根据坐标转成全景使用的百度墨卡托坐标系
    CGPoint point = BMKConvertToBaiduMercatorFromBD09LL(_coordinate);
    // 设定全景的pid， 这是指定显示某地的全景，也可以通过百度坐标进行显示全景
    [self.panoramaView setPanoramaWithX:point.x Y:point.y];

    [self.panoramaView addSubview:self.returnMapBtn];
}




#pragma mark - panorama view delegate
- (void)panoramaWillLoad:(BaiduPanoramaView *)panoramaView {

}

/**
 * @abstract 全景图加载完毕
 * @param panoramaView 当前全景视图
 * @param jsonStr 全景单点信息
 *
 */
- (void)panoramaDidLoad:(BaiduPanoramaView *)panoramaView descreption:(NSString *)jsonStr {
    // 不显示道路箭头
    [panoramaView showDirectionArrow:NO];
    // 关闭快速前进
    [panoramaView enableFastMoving:NO];
    // 百度墨卡托坐标
    CGPoint point = BMKConvertToBaiduMercatorFromBD09LL(_coordinate);
    // 添加全景overlay
    [panoramaView addImageOverlayById:@"1" X:point.x Y:point.y Z:0 image:[UIImage imageNamed:@"stop_new"]];
}

- (void)panoramaLoadFailed:(BaiduPanoramaView *)panoramaView error:(NSError *)error {
    NSLog(@"**********%@***************", error.userInfo);
}

- (void)panoramaView:(BaiduPanoramaView *)panoramaView overlayClicked:(NSString *)overlayId {
   
}

- (void)panoramaView:(BaiduPanoramaView *)panoramaView didReceivedMessage:(NSDictionary *)dict {
    // 旋转角度
    float heading = panoramaView.getPanoramaHeading;
    float mapAngle = self.mapView.getMapStatus.fRotation;
    if (mapAngle > 0) {
        heading = ((heading - mapAngle) / 180.0 * M_PI);
    } else {
        heading = (heading / 180.0 * M_PI);
    }
    [UIView animateWithDuration:0.1 animations:^{
        // 根据全景图旋转角度设置annotationView的角度，
        BMKAnnotationView *annotationView = [self.mapView viewForAnnotation:self.pointAnnotation];
        annotationView.transform = CGAffineTransformRotate(CGAffineTransformIdentity, heading);
    }];
}

#pragma mark - 地图delegate
- (BMKAnnotationView *)mapView:(BMKMapView *)mapView viewForAnnotation:(id<BMKAnnotation>)annotation {
    if ([annotation isKindOfClass:[BMKCustomPointAnnotation class]]) {
        BMKAnnotationView *view = [[BMKAnnotationView alloc] initWithAnnotation:annotation reuseIdentifier:@"BMKSCTXCustomAnnotation1"];
        view.image = [UIImage imageNamed:@"sportArrow"];
        view.enabled = NO;
        return view;
    }
    return [BMKAnnotationView new];
}

#pragma mark - 定位Delegate
- (void)BMKLocationManager:(BMKLocationManager *)manager didUpdateHeading:(CLHeading *)heading {
    if (!heading) {
        return;
    }
    if (!self.userLocation) {
        self.userLocation = [[BMKUserLocation alloc] init];
    }
    self.userLocation.heading = heading;
    [self.mapView updateLocationData:self.userLocation];
}

- (void)BMKLocationManager:(BMKLocationManager *)manager didUpdateLocation:(BMKLocation *)location orError:(NSError *)error {
    if (error) {
        NSLog(@"locError:{%ld - %@};", (long)error.code, error.localizedDescription);
    }
    if (!location) {
        return;
    }
    if (!self.userLocation) {
        self.userLocation = [[BMKUserLocation alloc] init];
    }
    self.userLocation.location = location.location;
    [self.mapView updateLocationData:self.userLocation];
//    [self.mapView setCenterCoordinate:self.userLocation.location.coordinate animated:YES];
}


#pragma mark - setter & getter
- (UIButton *)returnMapBtn {
    if (!_returnMapBtn) {
        _returnMapBtn = [UIButton buttonWithType:UIButtonTypeCustom];
        _returnMapBtn.layer.masksToBounds = YES;
        _returnMapBtn.frame = CGRectMake(CGRectGetWidth(self.panoramaView.frame) - 90, CGRectGetMaxY(self.mapView.frame) - 45, 80, 30);
        _returnMapBtn.layer.cornerRadius = 15;
        _returnMapBtn.backgroundColor = [UIColor whiteColor];
        _returnMapBtn.alpha = 0.6;
        [_returnMapBtn setTitle:@"返回地图" forState:UIControlStateNormal];
        [_returnMapBtn setTitleColor:[UIColor blackColor] forState:UIControlStateNormal];
        _returnMapBtn.hidden = YES;
        [_returnMapBtn addTarget:self action:@selector(mapBtnClick) forControlEvents:UIControlEventTouchUpInside];
    }
    return _returnMapBtn;
}

- (BMKLocationManager *)locManager {
    if (!_locManager) {
        _locManager = [[BMKLocationManager alloc] init];
        BMK_COORD_TYPE type = [BMKMapManager getCoordinateTypeUsedInBaiduMapSDK];
        if (BMK_COORDTYPE_COMMON == type) {
            _locManager.coordinateType = BMKLocationCoordinateTypeGCJ02;
        } else {
            _locManager.coordinateType = BMKLocationCoordinateTypeBMK09LL;
        }
        _locManager.delegate = self;
    }
    return _locManager;
}

- (BMKMapView *)mapView {
    if (!_mapView) {
        CGRect frame = CGRectMake(0, CGRectGetHeight(self.view.frame) - 280, CGRectGetWidth(self.view.frame), 280);
        _mapView = [[BMKMapView alloc] initWithFrame:frame];
        _mapView.mapType = BMKMapTypeStandard;
        _mapView.buildingsEnabled = FALSE;
        _mapView.delegate = self;
        _mapView.zoomLevel = 20;
        _mapView.userTrackingMode = BMKUserTrackingModeHeading;
        _mapView.showsUserLocation = YES;
    }
    return _mapView;
}

- (BMKCustomPointAnnotation *)pointAnnotation {
    if (!_pointAnnotation) {
        _pointAnnotation = [[BMKCustomPointAnnotation alloc] init];
        _pointAnnotation.coordinate = _coordinate;
    }
    return _pointAnnotation;
}

- (void)BMKLocationManager:(BMKLocationManager * _Nonnull)manager doRequestAlwaysAuthorization:(CLLocationManager * _Nonnull)locationManager {
    [locationManager requestAlwaysAuthorization];
}

- (void) dealloc {
    self.panoramaView = nil;
    self.panoramaView.delegate = nil;
}

- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];

    if (@available(iOS 13.0, *)) {
        self.navigationController.navigationBar.standardAppearance.backgroundColor = [UIColor clearColor];
        self.navigationController.navigationBar.scrollEdgeAppearance = nil;

    } else {
        // Fallback on earlier versions
    }
}

- (void)viewWillDisappear:(BOOL)animated {
    [super viewWillDisappear:animated];
    
    if (@available(iOS 13.0, *)) {
        self.navigationController.navigationBar.standardAppearance.backgroundColor = [UIColor whiteColor];
        self.navigationController.navigationBar.scrollEdgeAppearance = self.navigationController.navigationBar.standardAppearance;
    } else {
        
    }

}
@end
