//
//  BMKSearchViewController.m
//  PassengerDemo
//
//  Created by baidu on 2021/2/20.
//  Copyright © 2021 baidu. All rights reserved.
//

#import "BMKSearchViewController.h"
#import "BMKSugResultTableViewCell.h"
#import "BMKSearchView.h"
@interface BMKSearchViewController ()<UISearchBarDelegate, BMKSuggestionSearchDelegate, UITableViewDataSource, UITableViewDelegate, BMKSearchViewDelegate>
/// 自定义检索View
@property (nonatomic, strong) BMKSearchView *searchView;
/// 结果页
@property (nonatomic, strong) UITableView *tableView;
@property (nonatomic, copy) NSArray<BMKSuggestionInfo *> *sugResultInfos;
@end

@implementation BMKSearchViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    self.view.backgroundColor = [UIColor whiteColor];
    [self.view addSubview:self.searchView];
    [self.view addSubview:self.tableView];
    [self setupConstraints];
    
}

- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];
    [self.searchView.searchBar becomeFirstResponder];
}

- (UIView *)searchView {
    if (!_searchView) {
        _searchView = [[BMKSearchView alloc] init];
        _searchView.translatesAutoresizingMaskIntoConstraints = NO;
        _searchView.delegate = self;
    }
    return _searchView;
}

#pragma mark - BMKSearchViewDelegate
- (void)cancelSearch {
    [self.searchView endEditing:YES];
    [self dismissViewControllerAnimated:YES completion:nil];
}

- (void)textDidChange:(NSString *)searchText {
    [self searchSugInfoWithKey:searchText];
}

// 根据输入发送sug检索
- (void)searchSugInfoWithKey:(NSString *)key {
    if (key.length == 0) {
        //清空列表
        self.sugResultInfos = [NSArray array];
        [self.tableView reloadData];
        return;
    }
    
    BMKSuggestionSearchOption *option = [BMKSuggestionSearchOption new];
    option.keyword = key;
    option.cityname = self.searchView.cityInfoButton.titleLabel.text? : @"北京市";
    option.cityLimit = YES;
    
    BMKSuggestionSearch *sugSearch = [BMKSuggestionSearch new];
    sugSearch.delegate = self;
    [sugSearch suggestionSearch:option];

}

#pragma mark - BMKSuggestionSearchDelegate

/// 返回suggestion搜索结果
/// @param searcher 搜索对象
/// @param result  搜索结果
/// @param error BMKSearchErrorCode
- (void)onGetSuggestionResult:(BMKSuggestionSearch *)searcher result:(BMKSuggestionSearchResult *)result errorCode:(BMKSearchErrorCode)error {
    if (error == BMK_SEARCH_NO_ERROR) {
        self.sugResultInfos = [result.suggestionList copy];
        dispatch_async(dispatch_get_main_queue(), ^{
            [self.tableView reloadData];
        });
    }
}


- (void)setupConstraints {
    [self.view addConstraint:[NSLayoutConstraint constraintWithItem:self.searchView attribute:NSLayoutAttributeTop relatedBy:NSLayoutRelationEqual toItem:self.view attribute:NSLayoutAttributeTop multiplier:1 constant:[[UIApplication sharedApplication] statusBarFrame].size.height]];
    [self.view addConstraint:[NSLayoutConstraint constraintWithItem:self.searchView attribute:NSLayoutAttributeLeft relatedBy:NSLayoutRelationEqual toItem:self.view attribute:NSLayoutAttributeLeft multiplier:1 constant:0]];
    [self.view addConstraint:[NSLayoutConstraint constraintWithItem:self.searchView attribute:NSLayoutAttributeRight relatedBy:NSLayoutRelationEqual toItem:self.view attribute:NSLayoutAttributeRight multiplier:1 constant:0]];
    [self.view addConstraint:[NSLayoutConstraint constraintWithItem:self.searchView attribute:NSLayoutAttributeHeight relatedBy:NSLayoutRelationEqual toItem:nil attribute:NSLayoutAttributeNotAnAttribute multiplier:1 constant:50]];
    
    [self.view addConstraint:[NSLayoutConstraint constraintWithItem:self.tableView attribute:NSLayoutAttributeTop relatedBy:NSLayoutRelationEqual toItem:self.searchView attribute:NSLayoutAttributeBottom multiplier:1 constant:10]];
    [self.view addConstraint:[NSLayoutConstraint constraintWithItem:self.tableView attribute:NSLayoutAttributeBottom relatedBy:NSLayoutRelationEqual toItem:self.view attribute:NSLayoutAttributeBottom multiplier:1 constant:0]];
    [self.view addConstraint:[NSLayoutConstraint constraintWithItem:self.tableView attribute:NSLayoutAttributeLeft relatedBy:NSLayoutRelationEqual toItem:self.view attribute:NSLayoutAttributeLeft multiplier:1 constant:0]];
    [self.view addConstraint:[NSLayoutConstraint constraintWithItem:self.tableView attribute:NSLayoutAttributeRight relatedBy:NSLayoutRelationEqual toItem:self.view attribute:NSLayoutAttributeRight multiplier:1 constant:0]];
}

- (UITableView *)tableView {
    if (!_tableView) {
        _tableView = [[UITableView alloc] init];
        _tableView.delegate = self;
        _tableView.dataSource = self;
        _tableView.tableFooterView = [UIView new];
        _tableView.keyboardDismissMode = UIScrollViewKeyboardDismissModeOnDrag;
        _tableView.translatesAutoresizingMaskIntoConstraints = NO;
    }
    return _tableView;
}

#pragma mark - UITableViewDataSource

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return self.sugResultInfos.count;
}

- (CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath {
    BMKSuggestionInfo *info = self.sugResultInfos[indexPath.row];
    return 65 + 35 * ((info.children.count / 3) + (info.children.count % 3  > 0 ? 1:0));
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    static NSString *tipCellIdentifier = @"sugInfoList.cell";
    BMKSugResultTableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:tipCellIdentifier];
    if (cell == nil) {
        cell = [[BMKSugResultTableViewCell alloc] initWithStyle:UITableViewCellStyleSubtitle
                                      reuseIdentifier:tipCellIdentifier];
    }
    cell.suggestionInfo = self.sugResultInfos[indexPath.row];
    cell.completionHandler = ^(id  _Nullable info) {
        if ([info isKindOfClass:[BMKPoiInfo class]]) {
            if (self.completionHandler) {
                self.completionHandler(info);
            }
            [self.searchView endEditing:YES];
            [self dismissViewControllerAnimated:YES completion:nil];
        } else if ([info isKindOfClass:[BMKSuggestionInfo class]]) {
            self.searchView.searchBar.text = ((BMKSuggestionInfo *) info).key;
        }
        
    };
    return cell;
}


@end
