//
//  ViewController.m
//  BaiduRecommendStopDemo
//
//  Created by v_wangdachuan on 2022/11/14.
//

#import "ViewController.h"
#import "Masonry.h"
#import "BMKContentView.h"
#import "BMKSearchViewController.h"
#import "BMKCustomPaoPaoView.h"
#import "BMKCustomPointAnnotation.h"
#import "BMKPanoViewController.h"

@interface ViewController ()<BMKMapViewDelegate, BMKLocationManagerDelegate, BMKRecommendStopSearchDelegate, BMKGeoCodeSearchDelegate, BMKContentViewDelegate, BMKRouteSearchDelegate>

@property (nonatomic, strong) UIButton *customButton;

@property (nonatomic, strong) BMKLocationManager *locManager;

@property (nonatomic, strong) BMKMapView *mapView;

@property (nonatomic, strong) BMKCustomPointAnnotation *startPointAnnotation;
@property (nonatomic, strong) BMKCustomPointAnnotation *endPointAnnotation;

@property (nonatomic, strong) BMKCustomPointAnnotation *recommondStopAnnotation;

@property (nonatomic, strong) BMKCustomPaoPaoView *customPopView;

@property (nonatomic, strong) BMKRouteSearch *drivingRouteSearch;

@property (nonatomic, strong) BMKLocation *currentLocation;
@end

@implementation ViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
    self.title = @"全景推荐上车点";
    
    [self.locManager startUpdatingLocation];
    
    UIBarButtonItem *customButton = [[UIBarButtonItem alloc]initWithCustomView:self.customButton];
    self.navigationItem.rightBarButtonItem = customButton;
    
    [self.view addSubview:self.mapView];
    
    BMKContentView *view = [[BMKContentView alloc]init];
    view.delegate = self;
    [self.view addSubview:view];
    [view mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.view).offset(15);
        make.right.equalTo(self.view).offset(-15);
        make.height.mas_equalTo(110);
        make.bottom.equalTo(self.view).offset(-20);
    }];
}

- (void)viewWillDisappear:(BOOL)animated {
    [super viewWillDisappear:animated];
    [self.mapView viewWillDisappear];
}

- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];
    [self.mapView viewWillAppear];
}

// 测试使用
- (void)clickCustomButton {
    

    [self.mapView setCenterCoordinate:CLLocationCoordinate2DMake(39.915, 116.404) animated:YES];
    [self.mapView removeAnnotations:self.mapView.annotations];
    [self.mapView removeOverlays:self.mapView.overlays];
    
    self.startPointAnnotation = nil;
    self.endPointAnnotation = nil;
    self.recommondStopAnnotation = nil;

}

- (void)startEndInfoClicked:(UIButton *)sender {
    
    BMKSearchViewController *searchViewController = [[BMKSearchViewController alloc] init];
    searchViewController.cityName = @"全国";
    __weak typeof(self) weakSelf = self;
    searchViewController.completionHandler = ^(BMKPoiInfo * _Nonnull info) {
        __strong typeof(weakSelf)strongSelf = weakSelf;
        
        if (info) {
            if (sender.tag == 101) {
                // 起点
                [sender setTitle:[NSString stringWithFormat:@"从「%@」出发 >", info.name] forState:UIControlStateNormal];
                
                if (!strongSelf.startPointAnnotation) {
                    strongSelf.startPointAnnotation = [[BMKCustomPointAnnotation alloc] init];
                    strongSelf.startPointAnnotation.isLockedToScreen = YES;
                    strongSelf.startPointAnnotation.type = BMKCustomAnnotationTypeStart;
                    strongSelf.startPointAnnotation.screenPointToLock = CGPointMake(strongSelf.mapView.center.x, strongSelf.mapView.center.y);
                } else {
                    [strongSelf.mapView removeAnnotation:strongSelf.startPointAnnotation];
                }
                strongSelf.startPointAnnotation.title = info.name;
                [strongSelf.mapView addAnnotation:strongSelf.startPointAnnotation];
                [strongSelf.mapView setCenterCoordinate:info.pt animated:YES];
                
                // 先进行逆地理编码展示起点信息再去请求推荐上车点
                [self reverseGeoCodeSearch:self.startPointAnnotation.coordinate];
            } else {
                // 终点
                [sender setTitle:[NSString stringWithFormat:@"到「%@」下车 >", info.name] forState:UIControlStateNormal];
                
                if (strongSelf.startPointAnnotation) {
                    strongSelf.startPointAnnotation.isLockedToScreen = NO;
                }
                if (!strongSelf.endPointAnnotation) {
                    strongSelf.endPointAnnotation = [[BMKCustomPointAnnotation alloc] init];
                } else {
                    [strongSelf.mapView removeAnnotation:strongSelf.endPointAnnotation];
                }
                strongSelf.endPointAnnotation.coordinate = info.pt;
                strongSelf.endPointAnnotation.title = info.name;
                strongSelf.endPointAnnotation.type = BMKCustomAnnotationTypeEnd;
                [strongSelf.mapView addAnnotation:strongSelf.endPointAnnotation];
                [strongSelf carRoutePlan];
//                [strongSelf.mapView showAnnotations:strongSelf.mapView.annotations animated:YES];
            }
        }
    };
    searchViewController.modalPresentationStyle = UIModalPresentationFullScreen;
    [self presentViewController:searchViewController animated:YES completion:nil];
}

- (void)carRoutePlan {

    //初始化BMKRouteSearch实例
    _drivingRouteSearch = [[BMKRouteSearch alloc] init];
    //设置驾车路径的规划
    _drivingRouteSearch.delegate = self;
    /*
     线路检索节点信息类，一个路线检索节点可以通过经纬度坐标或城市名加地名确定
     */
    //实例化线路检索节点信息类对象
    BMKPlanNode *start = [[BMKPlanNode alloc]init];
    //起点名称
    start.name = self.startPointAnnotation.title;
    //起点所在城市，注：cityName和cityID同时指定时，优先使用cityID
    start.cityName = _currentLocation.rgcData.city;
    //起点坐标
    start.pt = self.startPointAnnotation.coordinate;
    //实例化线路检索节点信息类对象
    BMKPlanNode *end = [[BMKPlanNode alloc]init];
    //终点名称
    end.name = self.endPointAnnotation.title;
    //终点所在城市，注：cityName和cityID同时指定时，优先使用cityID
    end.cityName = _currentLocation.rgcData.city;
    //终点坐标
    end.pt = self.endPointAnnotation.coordinate;
    //初始化请求参数类BMKDrivingRoutePlanOption的实例
    BMKDrivingRoutePlanOption *drivingRoutePlanOption = [[BMKDrivingRoutePlanOption alloc]init];
    //检索的起点，可通过关键字、坐标两种方式指定。cityName和cityID同时指定时，优先使用cityID
    drivingRoutePlanOption.from = start;
    //检索的终点，可通过关键字、坐标两种方式指定。cityName和cityID同时指定时，优先使用cityID
    drivingRoutePlanOption.to = end;
    /*
     路线中每一个step的路况，默认使用BMK_DRIVING_REQUEST_TRAFFICE_TYPE_NONE
     BMK_DRIVING_REQUEST_TRAFFICE_TYPE_NONE：不带路况
     BMK_DRIVING_REQUEST_TRAFFICE_TYPE_PATH_AND_TRAFFICE：道路和路况
     */
    drivingRoutePlanOption.drivingRequestTrafficType = BMK_DRIVING_REQUEST_TRAFFICE_TYPE_PATH_AND_TRAFFICE;

    /**
     发起驾乘路线检索请求，异步函数，返回结果在BMKRouteSearchDelegate的onGetDrivingRouteResult中
     */
    BOOL flag = [_drivingRouteSearch drivingSearch:drivingRoutePlanOption];
    if (flag) {
        NSLog(@"驾车检索成功");
    } else {
        NSLog(@"驾车检索失败");
    }
}

// 逆地理编码
- (void)reverseGeoCodeSearch:(CLLocationCoordinate2D)coordinate {
    BMKGeoCodeSearch *search = [[BMKGeoCodeSearch alloc] init];
    search.delegate = self;
    BMKReverseGeoCodeSearchOption *reverseGeoCodeOption = [[BMKReverseGeoCodeSearchOption alloc]init];
    reverseGeoCodeOption.location = coordinate;
    // 是否访问最新版行政区划数据（仅对中国数据生效）
    reverseGeoCodeOption.isLatestAdmin = YES;
    BOOL flag = [search reverseGeoCode: reverseGeoCodeOption];
    if (flag) {
        NSLog(@"逆geo检索发送成功");
    }  else  {
        NSLog(@"逆geo检索发送失败");
    }
}


//请求推荐上车点信息
- (void)requestRecommendStopSearch:(CLLocationCoordinate2D)coordinate {
    // 初始化请求参数类BMKRecommendStopSearchOption的实例
    BMKRecommendStopSearchOption *option = [[BMKRecommendStopSearchOption alloc] init];
    // 推荐上车点经纬度 （必选）
    option.location = coordinate;
    [self searchData:option];
}

- (void)searchData:(BMKRecommendStopSearchOption *)option {
    // 初始化BMKRecommendStopSearch实例
    BMKRecommendStopSearch *search = [[BMKRecommendStopSearch alloc] init];
    // 推荐上车点检索的代理
    search.delegate = self;
    // 初始化请求参数类BMKRecommendStopSearchOption的实例
    BMKRecommendStopSearchOption *stopSearchOption = [[BMKRecommendStopSearchOption alloc] init];
    // 推荐上车点经纬度 （必选）
    stopSearchOption.location = option.location;
    /// 推荐上车点检索
    BOOL flag = [search recommendStopSearch:stopSearchOption];
    if (flag) {
        NSLog(@"推荐上车点检索成功");
    } else {
        NSLog(@"推荐上车点检索失败");
    }
}

/// 请求全景缩略图
- (void)requestPanoImageWithCoordinate:(BMKRecommendStopInfo *)info {
    
    NSString *urlString = [NSString stringWithFormat:@"https://api.map.baidu.com/panorama/v2?ak=输入您的AK&location=%f,%f&fov=180", info.location.longitude, info.location.latitude];
   
    NSURL *listURL = [NSURL URLWithString:urlString];
    NSURLSession * session= [NSURLSession sharedSession];
    NSURLSessionDataTask * dataTask= [session dataTaskWithURL:listURL completionHandler:^(NSData * _Nullable data, NSURLResponse * _Nullable response, NSError * _Nullable error) {
        
        if (!error) {
            dispatch_async(dispatch_get_main_queue(), ^{
                if ([UIImage imageWithData:data]) {
                    self.customPopView.panoImage = [UIImage imageWithData:data];
                }
            });
        }
        
    }];
    [dataTask resume];
}



#pragma mark - mapViewDelegate
/**
 返回驾车路线检索结果

 @param searcher 检索对象
 @param result 检索结果，类型为BMKDrivingRouteResult
 @param error 错误码 @see BMKSearchErrorCode
 */
- (void)onGetDrivingRouteResult:(BMKRouteSearch *)searcher result:(BMKDrivingRouteResult *)result errorCode:(BMKSearchErrorCode)error {
    
    if (error == BMK_SEARCH_NO_ERROR) {
        //+polylineWithPoints: count:坐标点的个数
        __block NSUInteger pointCount = 0;
        //获取所有驾车路线中第一条路线
        BMKDrivingRouteLine *routeline = (BMKDrivingRouteLine *)result.routes[0];
        NSMutableArray *trafficArray = [NSMutableArray array];
        //遍历驾车路线中的所有路段市
        [routeline.steps enumerateObjectsUsingBlock:^(id  _Nonnull obj, NSUInteger idx, BOOL * _Nonnull stop) {
            //获取驾车路线中的每条路段
            BMKDrivingStep *step = routeline.steps[idx];
            //统计路段所经过的地理坐标集合内点的个数
            pointCount += step.pointsCount;
            [trafficArray addObjectsFromArray:step.traffics];
        }];
        
        //+polylineWithPoints: count:指定的直角坐标点数组
        BMKMapPoint *points = new BMKMapPoint[pointCount];
        __block NSUInteger j = 0;
        //遍历驾车路线中的所有路段
        [routeline.steps enumerateObjectsUsingBlock:^(id  _Nonnull obj, NSUInteger idx, BOOL * _Nonnull stop) {
            //获取驾车路线中的每条路段
            BMKDrivingStep *step = routeline.steps[idx];
            //遍历每条路段所经过的地理坐标集合点
            for (NSUInteger i = 0; i < step.pointsCount; i ++) {
                //将每条路段所经过的地理坐标点赋值给points
                points[j].x = step.points[i].x;
                points[j].y = step.points[i].y;
                j ++;
            }
        }];
        //根据指定直角坐标点生成一段折线
//        BMKPolyline *polyline = [BMKPolyline polylineWithPoints:points count:pointCount];
        //根据指定直角坐标点生成多色折线
        BMKPolyline *polyline = [BMKPolyline polylineWithPoints:points count:pointCount textureIndex:trafficArray];
        
        /**
         向地图View添加Overlay，需要实现BMKMapViewDelegate的-mapView:viewForOverlay:方法
         来生成标注对应的View
         
         @param overlay 要添加的overlay
         */
        [self.mapView addOverlay:polyline];
        //根据polyline设置地图范围
        [self mapViewFitPolyline:polyline withMapView:self.mapView];
    }
}

//根据polyline设置地图范围
- (void)mapViewFitPolyline:(BMKPolyline *)polyline withMapView:(BMKMapView *)mapView {
    double leftTop_x, leftTop_y, rightBottom_x, rightBottom_y;
    if (polyline.pointCount < 1) {
        return;
    }
    BMKMapPoint pt = polyline.points[0];
    leftTop_x = pt.x;
    leftTop_y = pt.y;
    //左上方的点lefttop坐标（leftTop_x，leftTop_y）
    rightBottom_x = pt.x;
    rightBottom_y = pt.y;
    //右底部的点rightbottom坐标（rightBottom_x，rightBottom_y）
    for (int i = 1; i < polyline.pointCount; i++) {
        BMKMapPoint point = polyline.points[i];
        if (point.x < leftTop_x) {
            leftTop_x = point.x;
        }
        if (point.x > rightBottom_x) {
            rightBottom_x = point.x;
        }
        if (point.y < leftTop_y) {
            leftTop_y = point.y;
        }
        if (point.y > rightBottom_y) {
            rightBottom_y = point.y;
        }
    }
    BMKMapRect rect;
    rect.origin = BMKMapPointMake(leftTop_x, leftTop_y);
    rect.size = BMKMapSizeMake(rightBottom_x - leftTop_x, rightBottom_y - leftTop_y);
    UIEdgeInsets padding = UIEdgeInsetsMake(130 + 30, 80, 110 + 30 + 20, 80);
    [mapView fitVisibleMapRect:rect edgePadding:padding withAnimated:YES];
}



// 地图拖动
- (void)mapView:(BMKMapView *)mapView regionDidChangeAnimated:(BOOL)animated reason:(BMKRegionChangeReason)reason {
    
    if (self.startPointAnnotation && !self.startPointAnnotation.isLockedToScreen) return;
    
    //当起点固定在屏幕中的时候进行全景检索等服务，暂时先这样判断，有合理需求修改。
    if (self.startPointAnnotation.isLockedToScreen || !self.startPointAnnotation) {
        if (reason == BMKRegionChangeReasonGesture) {
            // 先进行逆地理编码，展示起点信息，然后再去请求推荐上车点
            for (BMKCustomPointAnnotation *annotation in mapView.annotations) {
                if (annotation.type == BMKCustomAnnotationTypeStart) {
                    self.startPointAnnotation = annotation;
                    [self reverseGeoCodeSearch:self.startPointAnnotation.coordinate];
                    break;
                }
            }
        }
    }
}


/// 推荐上车点检索结果回调
/// @param searcher 搜索对象
/// @param recommendStopResult  搜索结果
/// @param errorCode  错误号，@see BMKSearchErrorCode
- (void)onGetRecommendStopResult:(BMKRecommendStopSearch *)searcher result:(BMKRecommendStopSearchResult *)recommendStopResult errorCode:(BMKSearchErrorCode)errorCode {
    
    if (errorCode == BMK_SEARCH_NO_ERROR) {
        if (!recommendStopResult.recommendStopInfoList && recommendStopResult.recommendStopInfoList.count == 0) {
            return;
        }

        for (id<BMKAnnotation> annotation in self.mapView.annotations) {
            if (((BMKCustomPointAnnotation *)annotation).type == BMKCustomAnnotationTypeRecommendStop) {
                [self.mapView removeAnnotation:annotation];
            }
        }
        
        if (recommendStopResult.recommendStopInfoList.count > 0) {
            BMKRecommendStopInfo *info = [recommendStopResult.recommendStopInfoList firstObject];
            [self.mapView setCenterCoordinate:info.location animated:YES];
            self.customPopView.name = info.name;
            [self requestPanoImageWithCoordinate:info];
        }
        
        for (BMKRecommendStopInfo *info in recommendStopResult.recommendStopInfoList) {
            BMKCustomPointAnnotation *stopAnnotation = [[BMKCustomPointAnnotation alloc] init];
            stopAnnotation.type = BMKCustomAnnotationTypeRecommendStop;
            stopAnnotation.coordinate = info.location;
            stopAnnotation.title      = info.name;
            stopAnnotation.pid        = info.uid;

            [self.mapView addAnnotation:stopAnnotation];
        }
    }
}


// 逆地理编码回调
- (void)onGetReverseGeoCodeResult:(BMKGeoCodeSearch *)searcher result:(BMKReverseGeoCodeSearchResult *)result errorCode:(BMKSearchErrorCode)error {
    if (error == BMK_SEARCH_NO_ERROR) {
        //在此处理正常结果
        self.customPopView.name = result.sematicDescription;
        [self requestRecommendStopSearch:self.startPointAnnotation.coordinate];
    } else {
        NSLog(@"逆地理检索失败");
    }
}


// BMKAnnotationView回调
- (__kindof BMKAnnotationView *)mapView:(BMKMapView *)mapView viewForAnnotation:(id<BMKAnnotation>)annotation {
    if ([annotation isKindOfClass:[BMKCustomPointAnnotation class]]) {
        BMKAnnotationView *view = [[BMKAnnotationView alloc] initWithAnnotation:annotation reuseIdentifier:@"BMKSCTXCustomAnnotation"];
        if (annotation == self.startPointAnnotation) {
            view.image = [UIImage imageNamed:@"icon_start"];
            view.selected = YES;
            view.hidePaopaoWhenSingleTapOnMap = NO;
            view.centerOffset = CGPointMake(0, -(view.bounds.size.height*0.7));
            view.displayPriority = BMKFeatureDisplayPriorityDefaultHigh;
            //自定义paopaoview
            self.customPopView = [[BMKCustomPaoPaoView alloc] initWithFrame:CGRectMake(0, 0, 150, 70)];
            self.customPopView.name = annotation.title;
            self.customPopView.panoImage = ((BMKCustomPaoPaoView *)annotation).panoImage;
            __weak typeof(self) weakSelf = self;
            self.customPopView.block = ^{

                BMKPanoViewController *vc = [BMKPanoViewController new];
                vc.pid        = ((BMKCustomPointAnnotation *)annotation).pid;
                vc.coordinate = ((BMKCustomPointAnnotation *)annotation).coordinate;
                vc.text       = weakSelf.customPopView.name;
                [weakSelf.navigationController pushViewController:vc animated:YES];
            };
            
            BMKActionPaopaoView *pView = [[BMKActionPaopaoView alloc] initWithCustomView:self.customPopView];
            pView.frame = CGRectMake(0, 0, 150, 70);
            view.paopaoView = pView;
            
        } else if (annotation == self.endPointAnnotation) {
            view.image = [UIImage imageNamed:@"icon_end"];
            view.selected = YES;
            view.hidePaopaoWhenSingleTapOnMap = NO;
            view.centerOffset = CGPointMake(0, -(view.bounds.size.height * 0.7));
        } else {
            view.image = [UIImage imageNamed:@"recommendStop"];
            view.selected = NO;
            view.canShowCallout = NO;
            view.centerOffset = CGPointMake(0, -(view.bounds.size.height * 0.5));
        }
        return view;
    }
    return nil;
}

- (BMKOverlayView *)mapView:(BMKMapView *)mapView viewForOverlay:(id<BMKOverlay>)overlay {


    if ([overlay isKindOfClass:[BMKPolyline class]]) {
        //初始化一个overlay并返回相应的BMKPolylineView的实例
        BMKPolylineView *polylineView = [[BMKPolylineView alloc] initWithOverlay:overlay];
        ///路段的路况信息，成员为NSNumber。0：无数据；1：畅通；2：缓慢；3：拥堵 4：超级拥堵
        [polylineView loadStrokeTextureImages:@[
            [UIImage imageNamed:@"traffic_texture_unknown"],
            [UIImage imageNamed:@"traffic_texture_smooth"],
            [UIImage imageNamed:@"traffic_texture_slow"],
            [UIImage imageNamed:@"traffic_texture_congestion"],
            [UIImage imageNamed:@"traffic_texture_severe_congestion"]]];
        //设置polygonView的线宽度
        polylineView.lineWidth = 10.0;
        return polylineView;
    }
    return nil;
}

- (void)BMKLocationManager:(BMKLocationManager *)manager didUpdateLocation:(BMKLocation *)location orError:(NSError *)error {
    if (location) {
        _currentLocation = location;
    }
}

#pragma mark - setter & getter
- (BMKMapView *)mapView {
    if (!_mapView) {
        _mapView = [[BMKMapView alloc] initWithFrame:self.view.frame];
        _mapView.mapType = BMKMapTypeStandard;
        _mapView.buildingsEnabled = FALSE;
        _mapView.delegate = self;
        [_mapView setCenterCoordinate:CLLocationCoordinate2DMake(40.04808, 116.31287) animated:NO];
        _mapView.zoomLevel = 17;
        _mapView.userTrackingMode = BMKUserTrackingModeHeading;
        _mapView.showsUserLocation = YES;
        _mapView.ChangeWithTouchPointCenterEnabled = NO;
        _mapView.ChangeCenterWithDoubleTouchPointEnabled = NO;
    }
    return _mapView;
}

- (BMKLocationManager *)locManager {
    if (!_locManager) {
        _locManager = [[BMKLocationManager alloc] init];
        _locManager.coordinateType = BMKLocationCoordinateTypeBMK09LL;
        _locManager.delegate = self;
    }
    return _locManager;
}

- (UIButton *)customButton {
    if (!_customButton) {
        _customButton = [UIButton buttonWithType:UIButtonTypeRoundedRect];
        [_customButton setTitle:@"重置" forState:UIControlStateNormal];
        [_customButton.titleLabel setFont:[UIFont systemFontOfSize:17]];
        [_customButton setFrame:CGRectMake(0, 3, 69, 20)];
        [_customButton addTarget:self action:@selector(clickCustomButton) forControlEvents:UIControlEventTouchUpInside];
    }
    return _customButton;
}


@end
