//
//  BMKContentView.m
//  BaiduRecommendStopDemo
//
//  Created by v_wangdachuan on 2022/11/15.
//

#import "BMKContentView.h"
#import "Masonry.h"

#define kSpacing 20
#define kHeight 30

@interface BMKContentView ()
@property (nonatomic, strong) UIButton *startInfoButton;
@property (nonatomic, strong) UIButton *endInfoButton;
@end

@implementation BMKContentView

- (instancetype)initWithFrame:(CGRect)frame {
    self = [super initWithFrame:frame];
    if (self) {
        self.backgroundColor = [UIColor whiteColor];
        self.layer.cornerRadius = 5;
        self.translatesAutoresizingMaskIntoConstraints = NO;
        [self configUI];
    }
    return self;
}

- (void)configUI {
    
    [self addSubview:self.startInfoButton];
    [self addSubview:self.endInfoButton];
    
    [self.startInfoButton mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.top.equalTo(self).offset(kSpacing);
        make.right.equalTo(self).offset(-kSpacing);
        make.height.mas_equalTo(kHeight);
    }];
    
    [self.endInfoButton mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self).offset(kSpacing);
        make.right.bottom.equalTo(self).offset(-kSpacing);
        make.height.mas_equalTo(kHeight);
    }];
}

- (void)startEndInfoSearchAction:(UIButton *)sender {
    if (self.delegate && [self.delegate respondsToSelector:@selector(startEndInfoClicked:)]) {
        [self.delegate startEndInfoClicked:sender];
    }
}

- (UIButton *)startInfoButton {
    if (!_startInfoButton) {
        _startInfoButton = [[UIButton alloc] init];
        [_startInfoButton setTitle:@"请输入起点" forState:UIControlStateNormal];
        [_startInfoButton setImage:[UIImage imageNamed:@"search_start"] forState:UIControlStateNormal];
        [_startInfoButton setImage:[UIImage imageNamed:@"search_start"] forState:UIControlStateSelected];
        [_startInfoButton.imageView setContentMode:UIViewContentModeScaleAspectFit];
        [_startInfoButton setImageEdgeInsets:UIEdgeInsetsMake(6, 0, 6, 6)];
        [_startInfoButton setTitleColor:[UIColor blackColor] forState:UIControlStateNormal];
        [_startInfoButton.titleLabel setFont:[UIFont systemFontOfSize:14]];
        _startInfoButton.contentHorizontalAlignment = UIControlContentHorizontalAlignmentLeft;
        _startInfoButton.backgroundColor = [UIColor colorWithRed:239 /255.0 green:239 /255.0 blue:239 /255.0 alpha:1];
        _startInfoButton.translatesAutoresizingMaskIntoConstraints = NO;
        _startInfoButton.layer.cornerRadius = 5;
        [_startInfoButton addTarget:self action:@selector(startEndInfoSearchAction:) forControlEvents:UIControlEventTouchUpInside];
        _startInfoButton.titleLabel.lineBreakMode = NSLineBreakByTruncatingTail;
        _startInfoButton.tag = 101;
    }
    return _startInfoButton;
}

- (UIButton *)endInfoButton {
    if (!_endInfoButton) {
        _endInfoButton = [[UIButton alloc] init];
        [_endInfoButton setTitle:@"请输入终点" forState:UIControlStateNormal];
        [_endInfoButton setImage:[UIImage imageNamed:@"search_end"] forState:UIControlStateNormal];
        [_endInfoButton setImage:[UIImage imageNamed:@"search_end"] forState:UIControlStateSelected];
        [_endInfoButton.imageView setContentMode:UIViewContentModeScaleAspectFit];
        [_endInfoButton setImageEdgeInsets:UIEdgeInsetsMake(6, 0, 6, 6)];
        [_endInfoButton setTitleColor:[UIColor blackColor] forState:UIControlStateNormal];
        [_endInfoButton.titleLabel setFont:[UIFont systemFontOfSize:14]];
        _endInfoButton.contentHorizontalAlignment = UIControlContentHorizontalAlignmentLeft;
        _endInfoButton.translatesAutoresizingMaskIntoConstraints = NO;
        _endInfoButton.layer.cornerRadius = 5;
        _endInfoButton.backgroundColor = [UIColor colorWithRed:239 /255.0 green:239 /255.0 blue:239 /255.0 alpha:1];
        [_endInfoButton addTarget:self action:@selector(startEndInfoSearchAction:) forControlEvents:UIControlEventTouchUpInside];
        _endInfoButton.titleLabel.lineBreakMode = NSLineBreakByTruncatingTail;
        _endInfoButton.tag = 102;

    }
    return _endInfoButton;
}


@end
