//
//  BMKSCTXSearchView.m
//  SCTXPassengerDemo
//
//  Created by baidu on 2021/2/20.
//  Copyright © 2021 baidu. All rights reserved.
//

#import "BMKSearchView.h"
@interface BMKSearchView ()<UISearchBarDelegate, BMKLocationManagerDelegate>
/// 圆点
@property (nonatomic, strong) UIImageView *pointImageView;
/// 检索容器view
@property (nonatomic, strong) UIView *searchView;
/// 分割线
@property (nonatomic, strong) UIView *line;
/// 取消按钮
@property (nonatomic, strong) UIButton *cancelButton;

@property (nonatomic, strong) BMKLocationManager *locManager;
@end
@implementation BMKSearchView
- (instancetype)init {
    if (self = [super init]) {
        [self addSubview:self.pointImageView];
        [self addSubview:self.cancelButton];
        [self addSubview:self.searchView];
        [self.searchView addSubview:self.cityInfoButton];
        [self.searchView addSubview:self.line];
        [self.searchView addSubview:self.searchBar];
        self.searchBar.delegate = self;
        [self locationAction];
        
    }
    return self;
}

#pragma mark - UISearchBarDelegate
// 输入回调
- (void)searchBar:(UISearchBar *)searchBar textDidChange:(NSString *)searchText {
    if ([self.delegate respondsToSelector:@selector(textDidChange:)]) {
        [self.delegate textDidChange:searchText];
    }
}

// 确认检索
- (void)searchBarSearchButtonClicked:(UISearchBar *)searchBar {
    [self endEditing:YES];
}

- (UIImageView *)pointImageView {
    if (!_pointImageView) {
        _pointImageView = [[UIImageView alloc] init];
        [_pointImageView setImage:[UIImage imageNamed:@"darkPoint"]];
        _pointImageView.translatesAutoresizingMaskIntoConstraints = NO;
    }
    return _pointImageView;
}

- (UIView *)searchView {
    if (!_searchView) {
        _searchView = [[UIView alloc] init];
        _searchView.backgroundColor = [UIColor colorWithWhite:0.9 alpha:1];
        _searchView.layer.cornerRadius = 5;
        _searchView.translatesAutoresizingMaskIntoConstraints = NO;
    }
    return _searchView;
}

- (UIButton *)cityInfoButton {
    if (!_cityInfoButton) {
        _cityInfoButton = [[UIButton alloc] init];
        [_cityInfoButton setTitle:@"北京" forState:UIControlStateNormal];
        [_cityInfoButton.titleLabel setFont:[UIFont systemFontOfSize:15]];
        [_cityInfoButton setTitleColor:[UIColor blackColor] forState:UIControlStateNormal];
        [_cityInfoButton.titleLabel setLineBreakMode:NSLineBreakByTruncatingTail];
        _cityInfoButton.translatesAutoresizingMaskIntoConstraints = NO;
        [_cityInfoButton addTarget:self action:@selector(locationAction) forControlEvents:UIControlEventTouchUpInside];
    }
    return _cityInfoButton;
}

- (UISearchBar *)searchBar {
    if (!_searchBar) {
        _searchBar = [[UISearchBar alloc] init];
        _searchBar.translatesAutoresizingMaskIntoConstraints = NO;
        _searchBar.searchBarStyle = UISearchBarStyleMinimal;
        _searchBar.returnKeyType = UIReturnKeySearch;
        _searchBar.placeholder = @"请输入关键字";
        [_searchBar resignFirstResponder];
        // 隐藏search按钮
        [_searchBar setImage:[self getImageWithColor:[UIColor clearColor] height:30] forSearchBarIcon:UISearchBarIconSearch state:UIControlStateNormal];
        // 隐藏背景色
        _searchBar.backgroundImage = [self getImageWithColor:[UIColor clearColor] height:30];
        [_searchBar setSearchFieldBackgroundImage:[self getImageWithColor:[UIColor clearColor] height:30] forState:UIControlStateNormal];
        // 设置字体大小
        if (@available(iOS 13.0, *)) {
            [_searchBar.searchTextField setFont:[UIFont systemFontOfSize:15]];
        } else {
            UITextField *textField = (UITextField *)[self findViewWithClassName:@"UITextField" inView:_searchBar];
            [textField setFont:[UIFont systemFontOfSize:15]];
        }
        [_searchBar setTintColor:[UIColor blackColor]];
        [_searchBar setBarTintColor:[UIColor blackColor]];
    }
    return _searchBar;
}

- (UIView *)line {
    if (!_line) {
        _line = [[UIView alloc] init];
        [_line setBackgroundColor:[UIColor colorWithWhite:0.8 alpha:1]];
        _line.translatesAutoresizingMaskIntoConstraints = NO;
    }
    return _line;
}

- (UIButton *)cancelButton {
    if (!_cancelButton) {
        _cancelButton = [[UIButton alloc] init];
        [_cancelButton setTitle:@"取消" forState:UIControlStateNormal];
        [_cancelButton.titleLabel setFont:[UIFont systemFontOfSize:15]];
        [_cancelButton setTitleColor:[UIColor colorWithWhite:0.5 alpha:1.0] forState:UIControlStateNormal];
        [_cancelButton addTarget:self action:@selector(cancelAction) forControlEvents:UIControlEventTouchUpInside];
        _cancelButton.translatesAutoresizingMaskIntoConstraints = NO;
    }
    return _cancelButton;
}

- (void)cancelAction {
    if ([self.delegate respondsToSelector:@selector(cancelSearch)]) {
        [self.delegate cancelSearch];
    }
}

- (void)layoutSubviews {
    [super layoutSubviews];
    [self setupConstraints];
}

- (void)setupConstraints {
    [self addConstraint:[NSLayoutConstraint constraintWithItem:self.pointImageView attribute:NSLayoutAttributeCenterY relatedBy:NSLayoutRelationEqual toItem:self attribute:NSLayoutAttributeCenterY multiplier:1 constant:0]];
    [self addConstraint:[NSLayoutConstraint constraintWithItem:self.pointImageView attribute:NSLayoutAttributeLeft relatedBy:NSLayoutRelationEqual toItem:self attribute:NSLayoutAttributeLeft multiplier:1 constant:10]];
    [self addConstraint:[NSLayoutConstraint constraintWithItem:self.pointImageView attribute:NSLayoutAttributeWidth relatedBy:NSLayoutRelationEqual toItem:nil attribute:NSLayoutAttributeNotAnAttribute multiplier:1 constant:10]];
    [self addConstraint:[NSLayoutConstraint constraintWithItem:self.pointImageView attribute:NSLayoutAttributeHeight relatedBy:NSLayoutRelationEqual toItem:nil attribute:NSLayoutAttributeNotAnAttribute multiplier:1 constant:10]];
    
    [self addConstraint:[NSLayoutConstraint constraintWithItem:self.cancelButton attribute:NSLayoutAttributeTop relatedBy:NSLayoutRelationEqual toItem:self attribute:NSLayoutAttributeTop multiplier:1 constant:0]];
    [self addConstraint:[NSLayoutConstraint constraintWithItem:self.cancelButton attribute:NSLayoutAttributeBottom relatedBy:NSLayoutRelationEqual toItem:self attribute:NSLayoutAttributeBottom multiplier:1 constant:0]];
    [self addConstraint:[NSLayoutConstraint constraintWithItem:self.cancelButton attribute:NSLayoutAttributeWidth relatedBy:NSLayoutRelationEqual toItem:nil attribute:NSLayoutAttributeNotAnAttribute multiplier:1 constant:40]];
    [self addConstraint:[NSLayoutConstraint constraintWithItem:self.cancelButton attribute:NSLayoutAttributeRight relatedBy:NSLayoutRelationEqual toItem:self attribute:NSLayoutAttributeRight multiplier:1 constant:-10]];
    
    [self addConstraint:[NSLayoutConstraint constraintWithItem:self.searchView attribute:NSLayoutAttributeTop relatedBy:NSLayoutRelationEqual toItem:self attribute:NSLayoutAttributeTop multiplier:1 constant:10]];
    [self addConstraint:[NSLayoutConstraint constraintWithItem:self.searchView attribute:NSLayoutAttributeBottom relatedBy:NSLayoutRelationEqual toItem:self attribute:NSLayoutAttributeBottom multiplier:1 constant:-10]];
    [self addConstraint:[NSLayoutConstraint constraintWithItem:self.searchView attribute:NSLayoutAttributeLeft relatedBy:NSLayoutRelationEqual toItem:self attribute:NSLayoutAttributeLeft multiplier:1 constant:30]];
    [self addConstraint:[NSLayoutConstraint constraintWithItem:self.searchView attribute:NSLayoutAttributeRight relatedBy:NSLayoutRelationEqual toItem:self.cancelButton attribute:NSLayoutAttributeLeft multiplier:1 constant:-40]];
    
    [self.searchView addConstraint:[NSLayoutConstraint constraintWithItem:self.cityInfoButton attribute:NSLayoutAttributeTop relatedBy:NSLayoutRelationEqual toItem:self.searchView attribute:NSLayoutAttributeTop multiplier:1 constant:0]];
    [self.searchView addConstraint:[NSLayoutConstraint constraintWithItem:self.cityInfoButton attribute:NSLayoutAttributeBottom relatedBy:NSLayoutRelationEqual toItem:self.searchView attribute:NSLayoutAttributeBottom multiplier:1 constant:0]];
    [self.searchView addConstraint:[NSLayoutConstraint constraintWithItem:self.cityInfoButton attribute:NSLayoutAttributeLeft relatedBy:NSLayoutRelationEqual toItem:self.searchView attribute:NSLayoutAttributeLeft multiplier:1 constant:10]];
    [self.searchView addConstraint:[NSLayoutConstraint constraintWithItem:self.cityInfoButton attribute:NSLayoutAttributeWidth relatedBy:NSLayoutRelationLessThanOrEqual toItem:nil attribute:NSLayoutAttributeNotAnAttribute multiplier:1 constant:46]];
    
    [self.searchView addConstraint:[NSLayoutConstraint constraintWithItem:self.line attribute:NSLayoutAttributeTop relatedBy:NSLayoutRelationEqual toItem:self.searchView attribute:NSLayoutAttributeTop multiplier:1 constant:5]];
    [self.searchView addConstraint:[NSLayoutConstraint constraintWithItem:self.line attribute:NSLayoutAttributeBottom relatedBy:NSLayoutRelationEqual toItem:self.searchView attribute:NSLayoutAttributeBottom multiplier:1 constant:-5]];
    [self.searchView addConstraint:[NSLayoutConstraint constraintWithItem:self.line attribute:NSLayoutAttributeLeft relatedBy:NSLayoutRelationEqual toItem:self.cityInfoButton attribute:NSLayoutAttributeRight multiplier:1 constant:10]];
    [self.searchView addConstraint:[NSLayoutConstraint constraintWithItem:self.line attribute:NSLayoutAttributeWidth relatedBy:NSLayoutRelationEqual toItem:nil attribute:NSLayoutAttributeNotAnAttribute multiplier:1 constant:1]];
    
    [self.searchView addConstraint:[NSLayoutConstraint constraintWithItem:self.searchBar attribute:NSLayoutAttributeTop relatedBy:NSLayoutRelationEqual toItem:self.searchView attribute:NSLayoutAttributeTop multiplier:1 constant:0]];
    [self.searchView addConstraint:[NSLayoutConstraint constraintWithItem:self.searchBar attribute:NSLayoutAttributeBottom relatedBy:NSLayoutRelationEqual toItem:self.searchView attribute:NSLayoutAttributeBottom multiplier:1 constant:0]];
    [self.searchView addConstraint:[NSLayoutConstraint constraintWithItem:self.searchBar attribute:NSLayoutAttributeRight relatedBy:NSLayoutRelationEqual toItem:self.searchView attribute:NSLayoutAttributeRight multiplier:1 constant:0]];
    [self.searchView addConstraint:[NSLayoutConstraint constraintWithItem:self.searchBar attribute:NSLayoutAttributeLeft relatedBy:NSLayoutRelationEqual toItem:self.line attribute:NSLayoutAttributeLeft multiplier:1 constant:10]];
    
}

- (UIImage *)getImageWithColor:(UIColor *)color height:(CGFloat)height {
    // 描述矩形
    CGRect rect = CGRectMake(0.0f, 0.0f, 1.0f, height);
    // 开启位图上下文
    UIGraphicsBeginImageContext(rect.size);
    // 获取位图上下文
    CGContextRef context = UIGraphicsGetCurrentContext();
    // 使用color演示填充上下文
    CGContextSetFillColorWithColor(context, [color CGColor]);
    // 渲染上下文
    CGContextFillRect(context, rect);
    // 从上下文中获取图片
    UIImage *theImage = UIGraphicsGetImageFromCurrentImageContext();
    // 结束上下文
    UIGraphicsEndImageContext();

    return  theImage;
}

// 根据类名找到对应的子view
- (UIView *)findViewWithClassName:(NSString *)className inView:(UIView *)view{
    Class specificView = NSClassFromString(className);
    if ([view isKindOfClass:specificView]) {
        return view;
    }

    if (view.subviews.count > 0) {
        for (UIView *subView in view.subviews) {
            UIView *targetView = [self findViewWithClassName:className inView:subView];
            if (targetView != nil) {
                return targetView;
            }
        }
    }
    
    return nil;
}

- (BMKLocationManager *)locManager {
    if (!_locManager) {
        _locManager = [[BMKLocationManager alloc] init];
        BMK_COORD_TYPE type = [BMKMapManager getCoordinateTypeUsedInBaiduMapSDK];
        if (BMK_COORDTYPE_COMMON == type) {
            _locManager.coordinateType = BMKLocationCoordinateTypeGCJ02;
        } else {
            _locManager.coordinateType = BMKLocationCoordinateTypeBMK09LL;
        }
        _locManager.delegate = self;
    }
    return _locManager;
}

- (void)locationAction {
    __weak typeof(self.cityInfoButton) weakButton = self.cityInfoButton;
    [self.locManager requestLocationWithReGeocode:YES withNetworkState:YES completionBlock:^(BMKLocation * _Nullable location, BMKLocationNetworkState state, NSError * _Nullable error) {
        if (error) {
            NSLog(@"locError:{%ld - %@};", (long)error.code, error.localizedDescription);
        }
        if (!location) {
            return;
        }
        [weakButton setTitle:location.rgcData.city forState:UIControlStateNormal];
    }];
}
@end
